@@ -29,6 +29,7 @@ import types
 import uuid
 
 # third-party module(s)
+import dateutil.parser
 import requests
 
 # neocortix modules
@@ -48,7 +49,7 @@ class g_:
     signaled = False
     frameDetails = {}
     dataDirPath = None
-    deviceLocsWanted = True
+    #deviceLocsWanted = True
     frameProcessor = None
     framesToDo = collections.deque()
     nFramesWanted = None
@@ -347,7 +348,8 @@ def recruitInstance( launchedJsonFilePath, resultsLogFilePathIgnored ):
         deadline = min( g_.deadline, time.time() + args.instTimeLimit )
 
         rc = None
-        if g_.deviceLocsWanted:
+        #if g_.deviceLocsWanted:
+        if args.pushDeviceLocs:
             rc = pushDeviceLoc( inst )
             if rc:
                 logger.warning( 'rc from pushDeviceLoc was %d', rc )
@@ -516,23 +518,37 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
             jsonToKnownHosts.jsonToKnownHosts( startedInstances, khFile )
         
         goodInstances = []
-        if not g_.deviceLocsWanted:
-            goodInstances = startedInstances
-        elif startedInstances:
-            returnCodes = pushDeviceLocs( startedInstances )
+        badInstances = []
+        if startedInstances:
+            returnCodes = checkInstanceClocks( startedInstances, timeLimit=40 )
+            if not any( returnCodes ):
+                logger.info( 'checkInstanceClocks found all good' )
+                goodInstances = startedInstances
+            else:
+                logger.warning( 'checkInstanceClocks returned %s', returnCodes )
+                for index, rc in enumerate( returnCodes ):
+                    if rc == 0:
+                        goodInstances.append( startedInstances[index] )
+                    else:
+                        badInstances.append( startedInstances[index] )
+
+        if goodInstances and args.pushDeviceLocs:
+            pushedInstances = []
+            returnCodes = pushDeviceLocs( goodInstances )
             badInstances = []
             for index, rc in enumerate( returnCodes ):
                 if rc == 0:
-                    goodInstances.append( startedInstances[index] )
+                    pushedInstances.append( goodInstances[index] )
                 else:
-                    badInstances.append( startedInstances[index] )
-            if badInstances:
-                logger.info( 'could not pushDeviceLoc to %d instances', len(badInstances) )
-                badIids = [inst['instanceId'] for inst in badInstances ]
-                logOperation( 'terminateBad', badIids, '<recruitInstances>' )
-                terminateInstances( args.authToken, badIids )
-                logger.info( 'purging host keys')
-                purgeHostKeys( badInstances )
+                    badInstances.append( goodInstances[index] )
+            goodInstances = pushedInstances
+        if badInstances:
+            logger.info( 'could not prepare %d instances', len(badInstances) )
+            badIids = [inst['instanceId'] for inst in badInstances ]
+            logOperation( 'terminateBad', badIids, '<recruitInstances>' )
+            terminateInstances( args.authToken, badIids )
+            logger.info( 'purging host keys')
+            purgeHostKeys( badInstances )
         if not goodInstances:
             return []
         # install something on goodInstances, if required, else just return goodInstances
@@ -542,6 +558,8 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
             def waiter():
                 thr = threading.current_thread()
                 while not thr.stopRequested:
+                    if g_.interrupted:
+                        break
                     time.sleep( 30 )
                     if not thr.stopRequested:
                         logger.info( 'waiting for installer' )
@@ -902,6 +920,48 @@ def renderFramesOnInstance( inst ):
         saveProgress()
     return 0
 
+def stdCommandInstance( inst, cmd, timeLimit ):
+    sshSpecs = inst['ssh']
+    abbrevIid = inst['instanceId'][0:16]
+    #logInstallerOperation( iid, ['connect', sshSpecs['host'], sshSpecs['port']] )
+    returnCode = None
+    stderr=''
+    stdout=''
+    with subprocess.Popen(['ssh',
+                    '-p', str(sshSpecs['port']),
+                    '-o', 'ServerAliveInterval=360',
+                    '-o', 'ServerAliveCountMax=3',
+                    sshSpecs['user'] + '@' + sshSpecs['host'], cmd],
+                    encoding='utf8',
+                    stdout=subprocess.PIPE, stderr=subprocess.PIPE
+                    ) as proc:
+        try:
+            (stdout, stderr) = proc.communicate( timeout=timeLimit )
+            #stdout = stdout.decode('utf8')
+            #stderr = stderr.decode('utf8')
+            returnCode = proc.returncode
+        except subprocess.TimeoutExpired:
+            logger.warning( 'command took too long for instance %s', abbrevIid )
+            proc.terminate()  # just sends sigterm
+            returnCode = 124
+            try:
+                # give it a chance to respond to sigterm
+                proc.wait( timeout=timeLimit/2 )
+            except Exception as exc:
+                logger.warning( 'got exception after timeout (%s) %s', type(exc), exc )
+                proc.kill()  # sends sigkill as a last resort
+                logger.warning( 'killed process for %s', abbrevIid )
+        except Exception as exc:
+            logger.warning( 'ssh threw exception (%s) %s', type(exc), exc )
+            returnCode = -1
+    if returnCode:
+        logger.warning( 'ssh returnCode %d', returnCode )
+    # not sure if this is needed
+    if (returnCode == 1) and ('closed by remote host' in stderr):
+        returnCode = 255
+
+    return {'returnCode': returnCode, 'stdout': stdout, 'stderr': stderr}
+
 def commandInstance( inst, cmd, timeLimit ):
     deadline = time.time() + timeLimit
     sshSpecs = inst['ssh']
@@ -959,6 +1019,49 @@ def commandInstance( inst, cmd, timeLimit ):
             return 0
     return 1
 
+def checkInstanceClock( inst, timeLimit, pastMax=6.0, futureMax=3.0 ):
+    '''check clock on instance, return bad if off by too much'''
+    cmd = 'date --iso-8601=seconds'
+    result = stdCommandInstance( inst, cmd, timeLimit=timeLimit )
+    logger.info( 'result %s', result )
+    if result['returnCode']:
+        return result['returnCode']
+    masterDateTime = datetime.datetime.now( datetime.timezone.utc )  # tz-aware
+    nodeDateTime = dateutil.parser.parse( result['stdout'] )
+    delta = masterDateTime - nodeDateTime
+    discrep = delta.total_seconds()
+    iid = inst['instanceId']
+    logger.info( 'discrep: %.1f seconds on inst %s',
+        discrep, iid )
+    dMin = -abs(futureMax)
+    dMax = abs(pastMax)
+    if discrep > dMax or discrep < dMin:
+        logger.warning( 'bad time discrep: %.1f', discrep )
+        return 1
+    return 0
+
+
+def checkInstanceClocks( instances, timeLimit ):
+    '''check clocks on instances, in parallel'''
+    returnCodes = []
+    nInstances = len(instances)
+    with futures.ThreadPoolExecutor( max_workers=nInstances ) as executor:
+        parIter = executor.map( checkInstanceClock, instances, [timeLimit]*nInstances,
+            timeout=timeLimit )
+        returnCodes = [None] * len(instances)
+        try:
+            index = 0
+            for returnCode in parIter:
+                returnCodes[index] = returnCode
+                index += 1
+                time.sleep( .1 )
+        except KeyboardInterrupt:
+            logger.warning( 'interrupted, setting flag')
+            g_.interrupted = True
+            raise
+        logger.debug( 'returnCodes: %s', returnCodes )
+    return returnCodes
+
 def pushDeviceLoc( inst, timeLimit=30 ):
     iid = inst['instanceId']
     logger.debug( 'would push deviceLoc to instance %s', iid[0:16] )
@@ -978,6 +1081,7 @@ def pushDeviceLoc( inst, timeLimit=30 ):
         logFrameState( -1, 'pushDeviceLocDone', iid, rc )
     return rc
 
+
 def pushDeviceLocs( instances, timeLimit=40 ):
     '''push device-location info to instances, in parallel'''
     returnCodes = []
@@ -1265,6 +1369,7 @@ def createArgumentParser():
     ap.add_argument( '--commonInFilePath', help='a file to upload initially to all instances' )
     ap.add_argument( '--authToken', required=True, help='the NCS authorization token to use (required)' )
     ap.add_argument( '--outDataDir', help='output data darectory', default='./data/' )
+    ap.add_argument( '--pushDeviceLocs', type=boolArg, default=False, help='whether to store device location on each launched instance' )
     ap.add_argument( '--encryptFiles', type=boolArg, default=True, help='whether to encrypt files on launched instances' )
     ap.add_argument( '--filter', help='json to filter instances for launch' )
     ap.add_argument( '--frameTimeLimit', type=int, default=8*60*60, help='amount of time (in seconds) allowed for each frame' )
